import 'dart:convert';

import 'common/pagingquery.dart';
import 'common/pagingresult.dart';
import 'notifications/getsocialnotification.dart';
import 'notifications/notificationcontent.dart';
import 'notifications/notificationcontext.dart';
import 'notifications/queries/notificationsquery.dart';
import 'notifications/sendnotificationtarget.dart';
import 'src/native_bridge.dart';

class Notifications {
  /// Get the list of current user's notifications.
  ///
  /// [pagingQuery]   Query to filter notifications.
  static Future<PagingResult<GetSocialNotification>> get(
      PagingQuery<NotificationsQuery> pagingQuery) {
    return NativeBridge.async(
            'Notifications.get', jsonEncode(pagingQuery.toJSON()))
        .then((value) => new PagingResult.fromJSON(jsonDecode(value),
            (Map<String, dynamic> raw) => GetSocialNotification.fromJSON(raw)));
  }

  /// Get the number of current user's notifications based on the provided query.
  ///
  /// [query]   Query to filter notifications.
  static Future<int> getCount(NotificationsQuery query) {
    return NativeBridge.async('Notifications.count', jsonEncode(query.toJSON()))
        .then((result) => jsonDecode(result)['result']);
  }

  /// Set notification status.
  ///
  /// [newStatus]         New status.
  /// [notificationIds]   List of notification ids.
  static Future setStatus(String newStatus, List<String> notificationIds) {
    Map<String, dynamic> methodBody = {
      'status': newStatus,
      'notificationIds': notificationIds
    };
    return NativeBridge.async(
        'Notifications.setStatus', jsonEncode(methodBody));
  }

  /// Enable or disable receiving Push Notifications.
  /// [enabled] Enable or disable Push Notifications.
  static Future setPushNotificationsEnabled(bool enabled) {
    return NativeBridge.async('Notifications.setEnabled', jsonEncode(enabled));
  }

  /// Check if Push Notifications are enabled for current user.
  static Future<bool> arePushNotificationsEnabled() {
    return NativeBridge.async('Notifications.areEnabled', '').then((result) {
      return jsonDecode(result)['result'] == true;
    });
  }

  /// Send notification to any GetSocial user.
  ///
  /// [content]   Notification content to send.
  /// [target]    Notification recipients.
  static Future send(
      NotificationContent content, SendNotificationTarget target) {
    Map<String, dynamic> sendNotificationBody = {
      'target': target.toJSON(),
      'content': content.toJSON()
    };
    return NativeBridge.async(
        'Notifications.send', jsonEncode(sendNotificationBody));
  }

  /// Register device for Push Notifications.
  static Future registerDevice() {
    return NativeBridge.async('Notifications.registerDevice', '');
  }

  /// Set listener to be invoked when a Push Notification was clicked.
  ///
  /// [listener]  Function to be invoked.
  static void setOnNotificationClickedListener(
      Function(GetSocialNotification, NotificationContext) listener) {
    NativeBridge.registerListener(
        'Notifications.setOnNotificationClickedListener', (result) {
      var json = jsonDecode(result);
      listener(GetSocialNotification.fromJSON(json['notification']),
          NotificationContext.fromJSON(json['context']));
    });
  }

  /// Set listener to be invoked when a Push Notification was received.
  ///
  /// [listener]  Function to be invoked.
  static void setOnNotificationReceivedListener(
      Function(GetSocialNotification) listener) {
    NativeBridge.registerListener(
        'Notifications.setOnNotificationReceivedListener',
        (result) {
            listener(GetSocialNotification.fromJSON(jsonDecode(result)));
        }
        );
  }

  /// Set listener to be invoked when device was registered for Push Notifications.
  ///
  /// [listener]  Function to be invoked.
  static void setOnTokenReceivedListener(Function(String) listener) {
    NativeBridge.registerListener('Notifications.setOnTokenReceivedListener',
        (result) => listener(result));
  }
}
